using System;
using System.IO;
using System.Linq;
using System.Text;
using NUnit.Framework;

namespace Smdn.Formats.Thbgm {
  [TestFixture]
  public class ProductInfoTests {
    [Test]
    public void TestEmbeddedProductInfo()
    {
      Assert.IsNotNull(ProductInfo.Th06);
      Assert.IsNotNull(ProductInfo.Th07);
      Assert.IsNotNull(ProductInfo.Th08tr);
      Assert.IsNotNull(ProductInfo.Th08);
      Assert.IsNotNull(ProductInfo.Th09tr);
      Assert.IsNotNull(ProductInfo.Th09);
      Assert.IsNotNull(ProductInfo.Th095);
      Assert.IsNotNull(ProductInfo.Th10tr);
      Assert.IsNotNull(ProductInfo.Th10);
      Assert.IsNotNull(ProductInfo.Th10tr);
      Assert.IsNotNull(ProductInfo.Th10trWeb);
      Assert.IsNotNull(ProductInfo.Th11);
      Assert.IsNotNull(ProductInfo.Th11tr);
      Assert.IsNotNull(ProductInfo.Th11);
      Assert.IsNotNull(ProductInfo.Th12tr);
      Assert.IsNotNull(ProductInfo.Alcostg);

      Assert.AreEqual(17, ProductInfo.Th06.Tracks.Count,      "track count of Th06");
      Assert.AreEqual(20, ProductInfo.Th07.Tracks.Count,      "track count of Th07");
      Assert.AreEqual(7,  ProductInfo.Th08tr.Tracks.Count,    "track count of Th08tr");
      Assert.AreEqual(21, ProductInfo.Th08.Tracks.Count,      "track count of Th08");
      Assert.AreEqual(7,  ProductInfo.Th09tr.Tracks.Count,    "track count of Th09tr");
      Assert.AreEqual(19, ProductInfo.Th09.Tracks.Count,      "track count of Th09");
      Assert.AreEqual(6,  ProductInfo.Th095.Tracks.Count,     "track count of Th095");
      Assert.AreEqual(7,  ProductInfo.Th10tr.Tracks.Count,    "track count of Th10tr");
      Assert.AreEqual(8,  ProductInfo.Th10trWeb.Tracks.Count, "track count of Th10trWeb");
      Assert.AreEqual(18, ProductInfo.Th10.Tracks.Count,      "track count of Th10");
      Assert.AreEqual(8,  ProductInfo.Th11tr.Tracks.Count,    "track count of Th11tr");
      Assert.AreEqual(18, ProductInfo.Th11.Tracks.Count,      "track count of Th11");
      Assert.AreEqual(8,  ProductInfo.Th12tr.Tracks.Count,    "track count of Th12tr");
      Assert.AreEqual(18, ProductInfo.Th12.Tracks.Count,      "track count of Th12");

      foreach (var expected in new[] {
        new {Creator = ProductInfo.Th06.Creator,      Message = "creator of Th06"},
        new {Creator = ProductInfo.Th07.Creator,      Message = "creator of Th07"},
        new {Creator = ProductInfo.Th08.Creator,      Message = "creator of Th08"},
        new {Creator = ProductInfo.Th08tr.Creator,    Message = "creator of Th08tr"},
        new {Creator = ProductInfo.Th09.Creator,      Message = "creator of Th09"},
        new {Creator = ProductInfo.Th09tr.Creator,    Message = "creator of Th09tr"},
        new {Creator = ProductInfo.Th095.Creator,     Message = "creator of Th095"},
        new {Creator = ProductInfo.Th10.Creator,      Message = "creator of Th10"},
        new {Creator = ProductInfo.Th10tr.Creator,    Message = "creator of Th10tr"},
        new {Creator = ProductInfo.Th10trWeb.Creator, Message = "creator of Th10trWeb"},
        new {Creator = ProductInfo.Th11.Creator,      Message = "creator of Th11"},
        new {Creator = ProductInfo.Th11tr.Creator,    Message = "creator of Th11tr"},
        new {Creator = ProductInfo.Th12.Creator,      Message = "creator of Th12"},
        new {Creator = ProductInfo.Th12tr.Creator,    Message = "creator of Th12tr"},
      }) {
        Assert.AreEqual("上海アリス幻樂団", expected.Creator, expected.Message);
      }

      foreach (var product in ProductInfo.EmbeddedProducts) {
        Assert.IsTrue(product.IsEmbedded);
        Assert.IsNotNull(product.DefaultBgmSourcePath);
        Assert.IsNull(product.InstalledBgmSourcePath);
        Assert.AreEqual(product.DefaultBgmSourcePath, product.InstalledOrDefaultBgmSourcePath);

        Assert.IsTrue(Path.IsPathRooted(product.DefaultBgmSourcePath), "path must be rooted");

        if (Runtime.IsRunningOnWindows)
          Assert.IsTrue(product.DefaultBgmSourcePath.IndexOf(":\\") == 1, "path starts with X:\\");
        else
          Assert.IsTrue(product.DefaultBgmSourcePath.StartsWith("/"), "path starts with /");

        Assert.IsFalse(Path.GetFileName(product.DefaultBgmSourcePath).Contains(Path.DirectorySeparatorChar.ToString()), "must be file name");
      }

      foreach (var expected in new[] {
        new {Product = ProductInfo.Th06,      ShortName = "東方紅魔郷", AbbreviatedShortName = "紅魔郷", ShortestName = "紅", Prefix = "th06"},
        new {Product = ProductInfo.Th07,      ShortName = "東方妖々夢", AbbreviatedShortName = "妖々夢", ShortestName = "妖", Prefix = "th07"},
        new {Product = ProductInfo.Th08,      ShortName = "東方永夜抄", AbbreviatedShortName = "永夜抄", ShortestName = "永", Prefix = "th08"},
        new {Product = ProductInfo.Th08tr,    ShortName = "東方永夜抄", AbbreviatedShortName = "永夜抄", ShortestName = "永", Prefix = "th08tr"},
        new {Product = ProductInfo.Th09,      ShortName = "東方花映塚", AbbreviatedShortName = "花映塚", ShortestName = "花", Prefix = "th09"},
        new {Product = ProductInfo.Th09tr,    ShortName = "東方花映塚", AbbreviatedShortName = "花映塚", ShortestName = "花", Prefix = "th09tr"},
        new {Product = ProductInfo.Th095,     ShortName = "東方文花帖", AbbreviatedShortName = "文花帖", ShortestName = "文", Prefix = "th095"},
        new {Product = ProductInfo.Th10,      ShortName = "東方風神録", AbbreviatedShortName = "風神録", ShortestName = "風", Prefix = "th10"},
        new {Product = ProductInfo.Th10tr,    ShortName = "東方風神録", AbbreviatedShortName = "風神録", ShortestName = "風", Prefix = "th10tr"},
        new {Product = ProductInfo.Th10trWeb, ShortName = "東方風神録", AbbreviatedShortName = "風神録", ShortestName = "風", Prefix = "th10tr_web"},
        new {Product = ProductInfo.Th11,      ShortName = "東方地霊殿", AbbreviatedShortName = "地霊殿", ShortestName = "地", Prefix = "th11"},
        new {Product = ProductInfo.Th11tr,    ShortName = "東方地霊殿", AbbreviatedShortName = "地霊殿", ShortestName = "地", Prefix = "th11tr"},
        new {Product = ProductInfo.Th12,      ShortName = "東方星蓮船", AbbreviatedShortName = "星蓮船", ShortestName = "星", Prefix = "th12"},
        new {Product = ProductInfo.Th12tr,    ShortName = "東方星蓮船", AbbreviatedShortName = "星蓮船", ShortestName = "星", Prefix = "th12tr"},
      }) {
        Assert.AreEqual(expected.ShortName, expected.Product.ShortName, "ShortName of {0}", expected.Product.Title);
        Assert.AreEqual(expected.AbbreviatedShortName, expected.Product.AbbreviatedShortName, "AbbreviatedShortName of {0}", expected.Product.Title);
        Assert.AreEqual(expected.ShortestName, expected.Product.ShortestName, "ShortestName of {0}", expected.Product.Title);
        Assert.AreEqual(expected.Prefix, expected.Product.Prefix, "Prefix of {0}", expected.Product.Title);
      }

      foreach (var product in new[] {
        ProductInfo.Th06,
        ProductInfo.Th07,
        ProductInfo.Th08,
        ProductInfo.Th09,
        ProductInfo.Th095,
        ProductInfo.Th10,
        ProductInfo.Th11,
        ProductInfo.Th12,
        ProductInfo.Alcostg,
      }) {
        Assert.AreNotEqual(Guid.Empty, product.THxxBGMProductGuid);
      }

      Assert.IsTrue(ProductInfo.Th06.HasPerTrackBgmSource, "Th06.HasPerTrackBgmSource");

      foreach (var product in new[] {
        ProductInfo.Th07,
        ProductInfo.Th08,
        ProductInfo.Th09,
        ProductInfo.Th095,
        ProductInfo.Th10,
        ProductInfo.Th11,
        ProductInfo.Th12,
        ProductInfo.Alcostg,
      }) {
        Assert.IsFalse(product.HasPerTrackBgmSource, product.ShortName + " HasPerTrackBgmSource");
      }
    }

    [Test]
    public void TestGetUnifiedTitle()
    {
      Assert.AreEqual("東方妖々夢　～ Perfect Cherry Blossom.", ProductInfo.GetUnifiedTitle("東方妖々夢　～ Perfect Cherry Blossom.", TitleUnificationOptions.NoUnify));
      Assert.AreEqual("東方妖々夢  ～ Perfect Cherry Blossom.", ProductInfo.GetUnifiedTitle("東方妖々夢  ～ Perfect Cherry Blossom.", TitleUnificationOptions.NoUnify));
      Assert.AreEqual("東方妖々夢  ～ Perfect Cherry Blossom.", ProductInfo.GetUnifiedTitle("東方妖々夢　～ Perfect Cherry Blossom.", TitleUnificationOptions.DoubleSpace));
      Assert.AreEqual("東方妖々夢　～ Perfect Cherry Blossom.", ProductInfo.GetUnifiedTitle("東方妖々夢  ～ Perfect Cherry Blossom.", TitleUnificationOptions.SingleIdeographicSpace));

      Assert.AreEqual("東方妖々夢　\u301c Perfect Cherry Blossom.", ProductInfo.GetUnifiedTitle("東方妖々夢　\u301c Perfect Cherry Blossom.", TitleUnificationOptions.NoUnify));
      Assert.AreEqual("東方妖々夢　\uff5e Perfect Cherry Blossom.", ProductInfo.GetUnifiedTitle("東方妖々夢　\uff5e Perfect Cherry Blossom.", TitleUnificationOptions.NoUnify));

      Assert.AreEqual("東方妖々夢　〜 Perfect Cherry Blossom.", ProductInfo.GetUnifiedTitle("東方妖々夢　～ Perfect Cherry Blossom.", TitleUnificationOptions.WaveDash));
      Assert.AreEqual("東方妖々夢　～ Perfect Cherry Blossom.", ProductInfo.GetUnifiedTitle("東方妖々夢　〜 Perfect Cherry Blossom.", TitleUnificationOptions.FullWidthTilde));

      Assert.AreEqual("東方妖々夢\u0020\u0020\u301c Perfect Cherry Blossom.", ProductInfo.GetUnifiedTitle("東方妖々夢\u3000\uff5e Perfect Cherry Blossom.", TitleUnificationOptions.DoubleSpace | TitleUnificationOptions.WaveDash));
      Assert.AreEqual("東方妖々夢\u3000\uff5e Perfect Cherry Blossom.", ProductInfo.GetUnifiedTitle("東方妖々夢\u0020\u0020\u301c Perfect Cherry Blossom.", TitleUnificationOptions.SingleIdeographicSpace | TitleUnificationOptions.FullWidthTilde));
    }

    [Test]
    public void TestSearchProductInfoFiles()
    {
      const string dir = "test";

      try {
        if (Directory.Exists(dir))
          Directory.Delete(dir, true);

        Directory.CreateDirectory(dir);

        File.WriteAllText(Path.Combine(dir, "titles.txt"), string.Empty);
        File.WriteAllText(Path.Combine(dir, "titles1.txt"), string.Empty);
        File.WriteAllText(Path.Combine(dir, "titles_hoge.txt"), string.Empty);
        File.WriteAllText(Path.Combine(dir, "Titles_hoge.txt"), string.Empty);
        File.WriteAllText(Path.Combine(dir, "TITLES_hoge.txt"), string.Empty);
        File.WriteAllText(Path.Combine(dir, "titles_ignored"), string.Empty);
        File.WriteAllText(Path.Combine(dir, "titles_ignored.dat"), string.Empty);
        File.WriteAllText(Path.Combine(dir, "~titles_ignored.txt"), string.Empty);

        var foundFiles = ProductInfo.SearchProductInfoFiles(dir, SearchOption.TopDirectoryOnly).ToArray();

        if (Runtime.IsRunningOnWindows) {
          Assert.AreEqual(3, foundFiles.Length);

          Assert.IsTrue(foundFiles.Select(s => Path.GetFileName(s)).Contains("titles_hoge.txt", StringComparer.OrdinalIgnoreCase), "#1");
          Assert.IsTrue(foundFiles.Select(s => Path.GetFileName(s)).Contains("titles1.txt"), "#2");
          Assert.IsTrue(foundFiles.Select(s => Path.GetFileName(s)).Contains("titles.txt"), "#3");
        }
        else {
          Assert.AreEqual(5, foundFiles.Length);

          Assert.IsTrue(foundFiles.Select(s => Path.GetFileName(s)).Contains("titles_hoge.txt"), "#1");
          Assert.IsTrue(foundFiles.Select(s => Path.GetFileName(s)).Contains("Titles_hoge.txt"), "#2");
          Assert.IsTrue(foundFiles.Select(s => Path.GetFileName(s)).Contains("TITLES_hoge.txt"), "#3");
          Assert.IsTrue(foundFiles.Select(s => Path.GetFileName(s)).Contains("titles1.txt"), "#4");
          Assert.IsTrue(foundFiles.Select(s => Path.GetFileName(s)).Contains("titles.txt"), "#5");
        }
      }
      finally {
        if (Directory.Exists(dir))
          Directory.Delete(dir, true);
      }
    }

    [Test]
    public void TestGuessMissingInfoes()
    {
      var testfilename = "titles_th22tr.txt";

      try {
        if (File.Exists(testfilename))
          File.Delete(testfilename);

        File.WriteAllText(testfilename, @"@骨川アリス幻樂団\東方臑茶魔体験版\thbgm_tr.dat,東方臑茶魔　～ The Acquaintance Of My Daddy. 体験版
#=ProductInfo,Prefix,
99999999,99999999,99999999,
99999999,99999999,99999999,
99999999,99999999,99999999,スネ夫が激しい弾幕を展開する時に流れている曲
");

        var product = ProductInfo.LoadFrom(testfilename);

        Assert.IsFalse(product.IsEmbedded);
        Assert.AreEqual("骨川アリス幻樂団", product.Creator);
        Assert.AreEqual("th22tr", product.Prefix);
        Assert.AreEqual("東方臑茶魔", product.ShortName);
        Assert.AreEqual("臑茶魔", product.AbbreviatedShortName);
        Assert.AreEqual("臑", product.ShortestName);
      }
      finally {
        if (File.Exists(testfilename))
          File.Delete(testfilename);
      }
    }

    [Test]
    public void TestInstalledBgmSourcePathUpdated()
    {
      var testStreamFile = "test.thbgm.dat";
      var testTitleFile = "test.thbgm.txt";

      if (File.Exists(testStreamFile))
        File.Delete(testStreamFile);
      if (File.Exists(testTitleFile))
        File.Delete(testTitleFile);

      try {
        using (var stream = File.OpenWrite(testStreamFile)) {
          var writer = new BinaryWriter(stream);

          writer.Write(new byte[] {0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd, 0xcd});

          writer.Flush();
        }

        File.WriteAllText(testTitleFile, @"@thxx\thbgm.dat,testdata
#=ProductInfo,BgmSourceLength,8
0,4,4,
");

        var productInfo = ProductInfo.LoadFrom(testTitleFile);

        Assert.IsFalse(productInfo.IsEmbedded);

        Assert.IsNull(productInfo.InstalledBgmSourcePath);
        Assert.AreEqual(productInfo.DefaultBgmSourcePath, productInfo.InstalledOrDefaultBgmSourcePath);

        Assert.AreEqual(productInfo, ProductInfo.FindMatchedProduct(testStreamFile, new[] {productInfo}));

        Assert.AreEqual(testStreamFile, productInfo.InstalledBgmSourcePath);
        Assert.AreEqual(testStreamFile, productInfo.InstalledOrDefaultBgmSourcePath);
      }
      finally {
        if (File.Exists(testStreamFile))
          File.Delete(testStreamFile);
        if (File.Exists(testTitleFile))
          File.Delete(testTitleFile);
      }
    }

    [Test]
    public void TestLoadFrom_CharsetDetection_UTFWithBOM()
    {
      foreach (var pair in new[] {
        new {Name = "UTF8", Encoding = (Encoding)new UTF8Encoding(true)},
        new {Name = "UTF16-LE", Encoding = (Encoding)new UnicodeEncoding(false, true)},
        new {Name = "UTF16-BE", Encoding = (Encoding)new UnicodeEncoding(true, true)},
        new {Name = "UTF32-LE", Encoding = (Encoding)new UTF32Encoding(false, true)},
        new {Name = "UTF32-BE", Encoding = (Encoding)new UTF32Encoding(true, true)},
      }) {
        TestLoadFrom_CharsetDetection_東方(pair.Encoding, delegate(ProductInfo product) {
          Assert.AreEqual("東方臑茶魔", product.Title, pair.Name);
        });
      }
    }

    [Test]
    public void TestLoadFrom_CharsetDetection_UTF8WithoutBOM()
    {
      foreach (var pair in new[] {
        new {Name = "UTF16-LE", Encoding = (Encoding)new UnicodeEncoding(false, false)},
        new {Name = "UTF16-BE", Encoding = (Encoding)new UnicodeEncoding(true, false)},
        new {Name = "UTF32-LE", Encoding = (Encoding)new UTF32Encoding(false, false)},
        new {Name = "UTF32-BE", Encoding = (Encoding)new UTF32Encoding(true, false)},
      }) {
        var loaded = false;

        try {
          TestLoadFrom_CharsetDetection_東方(pair.Encoding, delegate(ProductInfo product) {
            loaded = true;

            Assert.AreNotEqual("東方臑茶魔", product.Title, pair.Name);
          });
        }
        catch (Exception ex) {
          if (loaded)
            Assert.Fail("unexcpected exception: {0}", ex);
          else
            Assert.Pass();
        }
      }
    }

    [Test]
    public void TestLoadFrom_CharsetDetection_東方_UTF8()
    {
      var utf8NoBom = new UTF8Encoding(false);

      TestLoadFrom_CharsetDetection_東方(utf8NoBom, delegate(ProductInfo product) {
        Assert.AreEqual("東方臑茶魔", product.Title);
      });
    }

    [Test]
    public void TestLoadFrom_CharsetDetection_東方_ShiftJIS()
    {
      TestLoadFrom_CharsetDetection_東方(Encoding.GetEncoding("Shift_JIS"), delegate(ProductInfo product) {
        Assert.AreEqual("東方臑茶魔", product.Title);
      });
    }

    [Test]
    public void TestLoadFrom_CharsetDetection_東方_EUCJP()
    {
      TestLoadFrom_CharsetDetection_東方(Encoding.GetEncoding("EUC-JP"), delegate(ProductInfo product) {
        Assert.AreNotEqual("東方臑茶魔", product.Title);
      });
    }

    private void TestLoadFrom_CharsetDetection_東方(Encoding charset, Action<ProductInfo> action)
    {
      var testTitleFile = "titles_thxx.txt";

      try {
        File.Delete(testTitleFile);

        File.WriteAllText(testTitleFile, @"@thxx\thbgm.dat,東方臑茶魔
#=ProductInfo,BgmSourceLength,8
0,4,4,
", charset);

        var product = ProductInfo.LoadFrom(testTitleFile);

        action(product);
      }
      finally {
        File.Delete(testTitleFile);
      }
    }

    [Test]
    public void TestLoadFrom_CharsetDetection_上海_UTF8()
    {
      var utf8NoBom = new UTF8Encoding(false);

      TestLoadFrom_CharsetDetection_上海(utf8NoBom, delegate(ProductInfo product) {
        Assert.AreEqual("タイトル", product.Tracks[0].Title);
      });
    }

    [Test]
    public void TestLoadFrom_CharsetDetection_上海_ShiftJIS()
    {
      TestLoadFrom_CharsetDetection_上海(Encoding.GetEncoding("Shift_JIS"), delegate(ProductInfo product) {
        Assert.AreEqual("タイトル", product.Tracks[0].Title);
      });
    }

    [Test]
    public void TestLoadFrom_CharsetDetection_上海_EUCJP()
    {
      TestLoadFrom_CharsetDetection_上海(Encoding.GetEncoding("EUC-JP"), delegate(ProductInfo product) {
        Assert.AreNotEqual("タイトル", product.Tracks[0].Title);
      });
    }

    private void TestLoadFrom_CharsetDetection_上海(Encoding charset, Action<ProductInfo> action)
    {
      var testTitleFile = "titles_thxx.txt";

      try {
        File.Delete(testTitleFile);

        File.WriteAllText(testTitleFile, @"@上海アリス幻樂団\thxx\thbgm.dat,thxx
#=ProductInfo,BgmSourceLength,8
0,4,4,タイトル
", charset);

        var product = ProductInfo.LoadFrom(testTitleFile);

        action(product);
      }
      finally {
        File.Delete(testTitleFile);
      }
    }

    [Test]
    public void TestLoadFrom_CharsetDetection_曲データ_UTF8()
    {
      var utf8NoBom = new UTF8Encoding(false);

      TestLoadFrom_CharsetDetection_曲データ(utf8NoBom, delegate(ProductInfo product) {
        Assert.AreEqual("タイトル", product.Tracks[0].Title);
      });
    }

    [Test]
    public void TestLoadFrom_CharsetDetection_曲データ_ShiftJIS()
    {
      TestLoadFrom_CharsetDetection_曲データ(Encoding.GetEncoding("Shift_JIS"), delegate(ProductInfo product) {
        Assert.AreEqual("タイトル", product.Tracks[0].Title);
      });
    }

    [Test]
    public void TestLoadFrom_CharsetDetection_曲データ_EUCJP()
    {
      TestLoadFrom_CharsetDetection_曲データ(Encoding.GetEncoding("EUC-JP"), delegate(ProductInfo product) {
        Assert.AreNotEqual("タイトル", product.Tracks[0].Title);
      });
    }

    private void TestLoadFrom_CharsetDetection_曲データ(Encoding charset, Action<ProductInfo> action)
    {
      var testTitleFile = "titles_thxx.txt";

      try {
        File.Delete(testTitleFile);

        File.WriteAllText(testTitleFile, @"#テスト用曲データ
@shanghai_alice\thxx\thbgm.dat,thxx
#=ProductInfo,BgmSourceLength,8
0,4,4,タイトル
", charset);

        var product = ProductInfo.LoadFrom(testTitleFile);

        action(product);
      }
      finally {
        File.Delete(testTitleFile);
      }
    }

    [Test]
    public void TestLoadFrom_CharsetDetection_DefaultCharset_JaJp()
    {
      var testTitleFile = "titles_thxx.txt";

      try {
        File.Delete(testTitleFile);

        File.WriteAllText(testTitleFile, @"@shanghai_alice\thxx\thbgm.dat,タイトル
#=ProductInfo,BgmSourceLength,8
0,4,4,
", Runtime.IsRunningOnWindows ? Encoding.GetEncoding("Shift_JIS") : new UTF8Encoding(false));

        var product = ProductInfo.LoadFrom(testTitleFile);

        Assert.AreEqual("タイトル", product.Title);
      }
      finally {
        File.Delete(testTitleFile);
      }
    }

    [Test]
    public void TestSaveToFile()
    {
      var testTitleFile = "test.thbgm.txt";

      if (File.Exists(testTitleFile))
        File.Delete(testTitleFile);

      try {
        ProductInfo.Th08.Save(testTitleFile);

        Assert.IsTrue(File.Exists(testTitleFile));

        var saved = ProductInfo.LoadFrom(testTitleFile);
        var expected = ProductInfo.Th08;

        Assert.IsNotNull(saved);

        Assert.AreEqual(expected.Title, saved.Title);
        Assert.AreEqual(expected.DefaultBgmSourcePath, saved.DefaultBgmSourcePath);
        Assert.AreEqual(expected.Creator, saved.Creator);
        Assert.AreEqual(expected.Prefix, saved.Prefix);
        Assert.AreEqual(expected.ShortName, saved.ShortName);
        Assert.AreEqual(expected.ShortestName, saved.ShortestName);
        Assert.AreEqual(expected.ReleaseDate, saved.ReleaseDate);
        Assert.AreEqual(expected.Tracks.Count, saved.Tracks.Count);

        for (var i = 0; i < expected.Tracks.Count; i++) {
          Assert.AreEqual(expected.Tracks[i].IntroOffset, saved.Tracks[i].IntroOffset, "intro offset of track {0}", i);
          Assert.AreEqual(expected.Tracks[i].IntroLength, saved.Tracks[i].IntroLength, "intro length of track {0}", i);
          Assert.AreEqual(expected.Tracks[i].RepeatLength, saved.Tracks[i].RepeatLength, "repeat length of track {0}", i);
          Assert.AreEqual(expected.Tracks[i].Title, saved.Tracks[i].Title, "title of track {0}", i);
        }
      }
      finally {
        if (File.Exists(testTitleFile))
          File.Delete(testTitleFile);
      }
    }

    [Test]
    public void TestSaveToFileProductHasPerTrackBgmSource()
    {
      var testTitleFile = "test.thbgm.txt";

      if (File.Exists(testTitleFile))
        File.Delete(testTitleFile);

      try {
        ProductInfo.Th06.Save(testTitleFile);

        Assert.IsTrue(File.Exists(testTitleFile));

        var saved = ProductInfo.LoadFrom(testTitleFile);
        var expected = ProductInfo.Th06;

        Assert.IsNotNull(saved);

        Assert.AreEqual(expected.Title, saved.Title);
        Assert.AreEqual(expected.DefaultBgmSourcePath, saved.DefaultBgmSourcePath);
        Assert.AreEqual(expected.Tracks.Count, saved.Tracks.Count);

        for (var i = 0; i < expected.Tracks.Count; i++) {
          Assert.AreEqual(expected.Tracks[i].IntroOffset, saved.Tracks[i].IntroOffset, "intro offset of track {0}", i);
          Assert.AreEqual(expected.Tracks[i].IntroLength, saved.Tracks[i].IntroLength, "intro length of track {0}", i);
          Assert.AreEqual(expected.Tracks[i].RepeatLength, saved.Tracks[i].RepeatLength, "repeat length of track {0}", i);
          Assert.AreEqual(expected.Tracks[i].Title, saved.Tracks[i].Title, "title of track {0}", i);
        }

        foreach (var line in File.ReadAllLines(testTitleFile)) {
          foreach (var expectedTrack in expected.Tracks) {
            if (line.Contains(expectedTrack.Title)) {
              StringAssert.StartsWith(string.Format("%th06_{0:D2}.wav,", expectedTrack.TrackNumber),
                                      line,
                                      "filename of track #" + expectedTrack.TrackNumber.ToString());
              StringAssert.Contains(saved.Tracks[expectedTrack.TrackNumber - 1].Title,
                                    line,
                                    "filename of track #" + expectedTrack.TrackNumber.ToString());
              break;
            }
          }
        }
      }
      finally {
        if (File.Exists(testTitleFile))
          File.Delete(testTitleFile);
      }
    }

    [Test]
    public void TestSaveOverwriteExistingFile()
    {
      var testTitleFile = "test.thbgm.txt";

      if (File.Exists(testTitleFile))
        File.Delete(testTitleFile);

      try {
        File.WriteAllText(testTitleFile, new string('*', 1024));

        ProductInfo.Th08.Save(testTitleFile);

        using (var outputStream = new MemoryStream()) {
          ProductInfo.Th08.Save(outputStream);

          outputStream.Position = 0L;

          Assert.AreEqual(outputStream.Length,
                          (new FileInfo(testTitleFile)).Length,
                          "output file length");

          // XXX
          var expected = (new StreamReader(outputStream, Encoding.UTF8)).ReadToEnd();
          var indexOfNonCommentLine = expected.IndexOf("\n@", StringComparison.Ordinal);

#if false
          Assert.AreEqual(expected,
                          File.ReadAllText(testTitleFile, Encoding.UTF8),
                          "output file content2);
#else
          Assert.AreEqual(expected.Substring(indexOfNonCommentLine),
                          File.ReadAllText(testTitleFile, Encoding.UTF8).Substring(indexOfNonCommentLine),
                          "output file content");
#endif
        }
      }
      finally {
        if (File.Exists(testTitleFile))
          File.Delete(testTitleFile);
      }
    }
  }
}