//  
//  Author:
//       smdn <smdn@smdn.jp>
// 
//  Copyright (c) 2004-2014 smdn
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

using System;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Windows.Forms;

using Smdn.Collections;

namespace Smdn.Applications.ThbgmExtractor {
  public partial class AboutDialog : Form {
    public AboutDialog()
    {
      InitializeComponent();

      var sb = new StringBuilder(1024);

      foreach (var pair in new[] {
        new {Assm = Assembly.GetEntryAssembly(), Url = "http://smdn.jp/works/tools/ThbgmExtractor/"},
        new {Assm = Assembly.GetAssembly(typeof(TagLib.Tag)), Url = "https://github.com/mono/taglib-sharp"},
      }) {
        sb.AppendFormat("{0} {1}", GetTitleFromAssembly(pair.Assm), pair.Assm.GetName().Version);
        sb.AppendLine();
        sb.AppendLine(GetCopyrightFromAssembly(pair.Assm));
        sb.AppendLine(pair.Url);
        sb.AppendLine();
      }

      sb.AppendLine();
      sb.AppendLine();

      foreach (var e in ThbgmExtractor.EncoderConfigs) {
        if (e == EncoderConfig.Empty)
          continue;

        sb.AppendFormat("[{0}]", e.Name);
        sb.AppendLine();

        string executableFullPath;
        FileVersionInfo versionInfo;

        if (e.GetExecutableFullPathAndVersion(out executableFullPath, out versionInfo)) {
          sb.AppendFormat("File:            {0}", versionInfo.FileName).AppendLine();
          sb.AppendFormat("FileDescription: {0}", versionInfo.FileDescription).AppendLine();
          sb.AppendFormat("FileVersion:     {0}", versionInfo.FileVersion).AppendLine();
          sb.AppendFormat("Product:         {0}", versionInfo.ProductName).AppendLine();
          sb.AppendFormat("ProductVersion:  {0}", versionInfo.ProductVersion).AppendLine();
        }
        else {
          sb.AppendLine("(executable file not found)");
        }

        sb.AppendLine();
      }

      textBoxAbout.Text = sb.ToString();
      textBoxAbout.Select(0, 0);
    }

    private string GetCopyrightFromAssembly(Assembly assm)
    {
      var copyright = assm.GetCustomAttributes(typeof(AssemblyCopyrightAttribute), false).FirstOrDefault() as AssemblyCopyrightAttribute;

      if (copyright == null)
        return string.Empty;
      else
        return copyright.Copyright;
    }

    private string GetTitleFromAssembly(Assembly assm)
    {
      var title = assm.GetCustomAttributes(typeof(AssemblyTitleAttribute), false).FirstOrDefault() as AssemblyTitleAttribute;

      if (title == null)
        return string.Empty;
      else
        return title.Title;
    }
  }
}
