//  
//  Author:
//       smdn <smdn@smdn.jp>
// 
//  Copyright (c) 2004-2017 smdn
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

using System;
using System.ComponentModel;
using System.Windows.Forms;

using Smdn.Formats;
using Smdn.Windows.Forms.ThbgmPlayer;

namespace Smdn.Applications.ThbgmExtractor {
  public class ExtendedTrackListViewItem : TrackListViewItem {
    internal ExtractBackgroundWorker.IExtractTrackProgress ExtractProgress {
      get; set;
    }

    public ExtendedTrackListViewItem()
    {
    }

    internal void InternalUpdateSubItems()
    {
      UpdateSubItems();
    }

    protected override void OnEffectPropertyChanged(PropertyChangedEventArgs e)
    {
      var listView = ListView as ExtendedTrackListView;

      if (listView != null)
        listView.OnTrackEffectPropertyChanged(e);

      base.OnEffectPropertyChanged(e);
    }

    protected override string GetSubItemText(TrackListViewColumnHeader header)
    {
      switch (header.BindingValueName) {
        case "OutputFileLength":
          return string.Format(SIPrefixFormat.CurrentInfo,
                               "{0:f}",
                               Track.TrackInfo.StreamFormat.ToAlignedByteCount(Track.Effect.TotalLength));

        case "ExtractProgress":
          if (ExtractProgress == null)
            return string.Empty;
          else
            return string.Format("{0:P1} {1}", ExtractProgress.ProgressPercent, ExtractProgress.StatusText);

        default:
          return base.GetSubItemText(header);
      }
    }
  }
}
