//  
//  Author:
//       smdn <smdn@smdn.jp>
// 
//  Copyright (c) 2004-2014 smdn
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

using System;
using System.IO;
using System.Linq;
using System.Xml;

using Smdn.Collections;
using Smdn.Formats.Thbgm;
using Smdn.Xml;

namespace Smdn.Applications.ThbgmExtractor {
  public static class THxxBGM {
    public const string THxxBGMSettingsFileName = "setting.xml";

    public static void ImportTHxxBGMSettings(string file)
    {
      var xml = new XmlDocument();

      xml.Load(file);

      // /THxxBGMSettings/BgmPath/Count
      var count = xml.GetSingleNodeValueOf<int>("/THxxBGMSettings/BgmPath/Count/text()", 0, Convert.ToInt32);

      for (var n = 1; n <= count; n++) { // 1-based
        // /THxxBGMSettings/BgmPath/Item[n]
        var itemNode = xml.SelectSingleNode(string.Format("/THxxBGMSettings/BgmPath/Item{0}", n));

        if (itemNode == null)
          continue;

        if (itemNode.GetSingleNodeValueOf<int>("Use/text()", 0, Convert.ToInt32) != 1)
          // no use, ignore
          continue;

        var guid = itemNode.GetSingleNodeValueOf<Guid>("Guid/text()", Guid.Empty, XmlConvert.ToGuid);

        var product = ThbgmExtractor.Products.FirstOrDefault(delegate(ProductInfo p) {
          return p.THxxBGMProductGuid == guid;
        });

        if (product == null)
          // unsupported or not found
          return;

        var bgmPath = itemNode.GetSingleNodeValueOf("BgmPath/text()");

        bgmPath = Path.Combine(bgmPath, Path.GetFileName(product.DefaultBgmSourcePath));

        if (ProductInfo.FindMatchedProduct(bgmPath, ThbgmExtractor.Products) != product) // InstalledBgmSourcePath will be updated
          continue;

        Console.WriteLine("imported installed bgm source path: {0} => {1}", product.Title, product.InstalledBgmSourcePath);
      }
    }
  }
}
