//  
//  Author:
//       smdn <smdn@smdn.jp>
// 
//  Copyright (c) 2004-2014 smdn
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;

using Smdn.Collections;
using Smdn.Formats.Playlists;
using Smdn.Windows.Forms.ThbgmPlayer;

namespace Smdn.Applications.ThbgmExtractor {
  public static class TrackPlaylist {
    public static void Save(string pathPlaylist, IDictionary<Track, string> trackAndPaths, PlaylistFormat format)
    {
      var playlistBaseDirectory = Path.GetDirectoryName(pathPlaylist);
      var product = trackAndPaths.Keys.First().TrackInfo.Product;

      var playlist = new Playlist(trackAndPaths.Select(delegate(KeyValuePair<Track, string> pair) {
        Uri trackLocation;

        if (Path.GetDirectoryName(pair.Value).StartsWith(playlistBaseDirectory))
          // set relative path
          //trackLocation = new Uri(new Uri(string.Format("file://{0}", playlistBaseDirectory)), "./" + pair.Value.Replace(playlistBaseDirectory, string.Empty));
          trackLocation = new Uri("." + pair.Value.Replace(playlistBaseDirectory, string.Empty), UriKind.Relative);
        else
          // set absolute path
          trackLocation = new Uri(string.Format("file://{0}",pair.Value));

        var entry = new PlaylistEntry(trackLocation);
        var track = pair.Key;

        entry.Title = track.TrackInfo.Title;
        entry.TrackNumber = track.TrackInfo.TrackNumber;
        entry.AlbumTitle = track.TrackInfo.Product.Title;
        entry.Duration = track.Effect.TotalLength;

        return entry;
      }));

      playlist.Creator = product.Creator;
      playlist.CreationDate = DateTimeOffset.Now;
      playlist.Generator = ThbgmExtractor.VersionString;
      playlist.Title = product.Title;

      playlist.Save(pathPlaylist, format);
    }
  }
}
