//  
//  Author:
//       smdn <smdn@smdn.jp>
// 
//  Copyright (c) 2004-2014 smdn
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

using System;
using System.Reflection;
using System.Collections.Generic;
using System.Windows.Forms;

using Smdn.Windows.Forms;
using Smdn.Windows.Forms.ThbgmPlayer;

namespace Smdn.Applications.ThbgmExtractor {
  public class TrackTagListView : EditableListView {
    public TrackTagListView()
      : base()
    {
      //CheckBoxes = true;
      HeaderStyle = ColumnHeaderStyle.Nonclickable;
      MultiSelect = false;
      FullRowSelect = true;
      HideSelection = false;

      Columns.Add(new TrackTagListViewColumnHeader("TrackNumber", Texts.GetText("TrackNumber"), 45));
#if DEBUG
      Columns.Add(new TrackTagListViewColumnHeader("IsDirty", "IsDirty", 20));
#endif
      Columns.Add(new TrackTagListViewColumnHeader("Title", ThbgmExtractor.GetText("TrackTag_Title"), 260));
      Columns.Add(new TrackTagListViewColumnHeader("OutputFilename", ThbgmExtractor.GetText("OutputFilename"), 200));
      Columns.Add(new TrackTagListViewColumnHeader("Album", ThbgmExtractor.GetText("TrackTag_Album"), 260));
      Columns.Add(new TrackTagListViewColumnHeader("Performers", ThbgmExtractor.GetText("TrackTag_Performers"), 120));
      Columns.Add(new TrackTagListViewColumnHeader("AlbumArtists", ThbgmExtractor.GetText("TrackTag_AlbumArtists"), 100));
      Columns.Add(new TrackTagListViewColumnHeader("Composers", ThbgmExtractor.GetText("TrackTag_Composers"), 100));
      Columns.Add(new TrackTagListViewColumnHeader("Genres", ThbgmExtractor.GetText("TrackTag_Genres"), 60));
      Columns.Add(new TrackTagListViewColumnHeader("Comment", ThbgmExtractor.GetText("TrackTag_Comment"), 200));
    }

    public void SetTrackTags(IEnumerable<TrackTag> trackTags)
    {
      BeginUpdate();

      Items.Clear();

      foreach (var trackTag in trackTags) {
        var item = new TrackTagListViewItem();

        Items.Add(item);

        item.TrackTag = trackTag;
      }

      EndUpdate();
    }

    protected override ListViewSubItemEditorBase CreateSubItemEditor(ListViewItem item, ListViewItem.ListViewSubItem subItem)
    {
      return CreateSubItemEditor(item as TrackTagListViewItem,
                                 Columns[item.SubItems.IndexOf(subItem)] as TrackTagListViewColumnHeader,
                                 subItem);
    }

    private class TrackTagTextBoxListViewSubItemEditor : TextBoxListViewSubItemEditor {
      public TrackTagTextBoxListViewSubItemEditor(ListViewItem item, ListViewItem.ListViewSubItem subItem, string bindingValueName)
        : base(item, subItem)
      {
        this.bindingValueName = bindingValueName;
      }

      protected override Control BeginEdit()
      {
        var trackTag = (Item as TrackTagListViewItem).TrackTag;

        switch (bindingValueName) {
          case "OutputFilename":
            EditControl.Text = trackTag.OutputFileName;
            break;

          default: {
            var bindingProperty = trackTag.GetType().GetProperty(bindingValueName, BindingFlags.Public | BindingFlags.Instance | BindingFlags.DeclaredOnly);

            if (bindingProperty.PropertyType == typeof(string[]))
              bindingProperty = trackTag.GetType().GetProperty("Joined" + bindingValueName);

            EditControl.Text = (string)bindingProperty.GetValue(trackTag, null);

            break;
          }
        }

        EditControl.SelectAll();

        return EditControl;
      }

      protected override void OnEditLeave(EventArgs e)
      {
        base.OnEditLeave(e);

        OnEditEntered();
      }

      protected override void OnEditEntered(EventArgs e)
      {
        base.OnEditEntered(e);

        OnEditEntered();
      }

      private void OnEditEntered()
      {
        var item = Item as TrackTagListViewItem;
        var trackTag = item.TrackTag;

        switch (bindingValueName) {
          case "OutputFilename":
            trackTag.OutputFileName = EditControl.Text;
            SubItem.Text = trackTag.FormattedOutputFileName;
            break;

          default: {
            var bindingProperty = trackTag.GetType().GetProperty(bindingValueName, BindingFlags.Public | BindingFlags.Instance | BindingFlags.DeclaredOnly);

            if (bindingProperty.PropertyType == typeof(string))
              bindingProperty.SetValue(trackTag, EditControl.Text, null);
            else if (bindingProperty.PropertyType == typeof(string[]))
              bindingProperty.SetValue(trackTag, TrackTag.GetSplitted(EditControl.Text), null);

            break;
          }
        }
      }

      private string bindingValueName;
    }

    protected virtual ListViewSubItemEditorBase CreateSubItemEditor(TrackTagListViewItem item, TrackTagListViewColumnHeader header, ListViewItem.ListViewSubItem subItem)
    {
      switch (header.BindingValueName) {
        case "OutputFilename":
        case "Title":
        case "Album":
        case "Performers":
        case "AlbumArtists":
        case "Composers":
        case "Genres":
        case "Comment":
          return new TrackTagTextBoxListViewSubItemEditor(item, subItem, header.BindingValueName);

        case "TrackNumber":
        case "IsDirty":
        default:
          return null;
      }
    }
  }
}
