//  
//  Author:
//       smdn <smdn@smdn.jp>
// 
//  Copyright (c) 2004-2014 smdn
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

using System;
using System.ComponentModel;
using System.Windows.Forms;

namespace Smdn.Applications.ThbgmExtractor {
  public class TrackTagListViewItem : ListViewItem {
    public TrackTag TrackTag {
      get { return trackTag; }
      set
      {
        if (value == null)
          throw new ArgumentNullException();

        if (trackTag != null)
          trackTag.PropertyChanged -= TrackTag_PropertyChanged;

        trackTag = value;
        trackTag.PropertyChanged += TrackTag_PropertyChanged;

        UpdateSubItems();
      }
    }

    protected internal virtual void UpdateSubItems()
    {
      if (SubItems.Count != ListView.Columns.Count) {
        SubItems.Clear();

        for (var i = 0; i < ListView.Columns.Count; i++) {
          SubItems.Add(string.Empty);
        }
      }

      for (var i = 0; i < ListView.Columns.Count; i++) {
        SubItems[i].Text = GetSubItemText(ListView.Columns[i] as TrackTagListViewColumnHeader);
      }
    }

    protected virtual string GetSubItemText(TrackTagListViewColumnHeader header)
    {
      switch (header.BindingValueName) {
        case "TrackNumber":             return TrackTag.TrackInfo.TrackNumber.ToString();
        case "IsDirty":                 return TrackTag.IsDirty.ToString();

        case "Title":           return TrackTag.Title;
        case "OutputFilename":  return TrackTag.FormattedOutputFileName;
        case "Album":           return TrackTag.Album;
        case "Performers":      return TrackTag.JoinedPerformers;
        case "AlbumArtists":    return TrackTag.JoinedAlbumArtists;
        case "Composers":       return TrackTag.JoinedComposers;
        case "Genres":          return TrackTag.JoinedGenres;
        case "Comment":         return TrackTag.Comment;
        default: return string.Empty;
      }
    }

    private void TrackTag_PropertyChanged(object sender, PropertyChangedEventArgs e)
    {
      if (sender == trackTag)
        OnTrackTagPropertyChanged(e);
    }

    protected virtual void OnTrackTagPropertyChanged(PropertyChangedEventArgs e)
    {
      UpdateSubItems();
    }

    private TrackTag trackTag;
  }
}
